//
//  MulticastDaemon.m
//  WrestlingTools
//
//  Created by Adam Fenn on 11/15/11.
//  Copyright (c) Wrestling Video Solutions. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "MulticastDaemon.h"
#include <sys/types.h>  /* for type definitions */
#include <sys/socket.h> /* for socket API calls */
#include <netinet/in.h> /* for address structs */
#include <arpa/inet.h>  /* for sockaddr_in */
#include <stdio.h>      /* for printf() and fprintf() */
#include <stdlib.h>     /* for atoi() */
#include <string.h>     /* for strlen() */
#include <unistd.h>     /* for close() */

@implementation MulticastDaemon

- (void)doWorkWithDelegate:(id<ThreadLogicContainerDelegate>)delegate
{
    @autoreleasepool
    {
        [delegate multicastDaemonDidStart:self];

        int sock;                     /* socket descriptor */
        int flag_on = 1;              /* socket option flag */
        struct sockaddr_in mc_addr;   /* socket address structure */
        char recv_str[MAX_LEN+1];     /* buffer to receive string */
        int recv_len;                 /* length of string received */
        struct ip_mreq mc_req;        /* multicast request structure */
        const char* mc_addr_str;            /* multicast IP address */
        unsigned short mc_port;       /* multicast port */
        struct sockaddr_in from_addr; /* packet source */
        unsigned int from_len;        /* source addr length */
        NSString *receivedString;
        int error = 0;                /* if we had an error in binding or anything */
        
        //mc_addr_str = "225.0.0.0";      /* arg 1: multicast ip address */
        //mc_port = 50000;    /* arg 2: multicast port number */
        
        mc_addr_str = [[delegate multicastAddress] UTF8String];
        mc_port = [[delegate portNumber] intValue];
        
        //NSLog(@"ipaddr: %s", mc_addr_str);
        //NSLog(@"portnum: %i", mc_port);
        
        /* create socket to join multicast group on */
        if ((sock = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0) {
            [delegate multicastDaemonDidFinish:self withError:@"socket() failed"];
            error = 1;
        }
        
        /* set reuse port to on to allow multiple binds per host */
        if ((setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &flag_on,
                        sizeof(flag_on))) < 0) {
            [delegate multicastDaemonDidFinish:self withError:@"setsockopt() failed"];

            error = 1;
        }
        
        /* construct a multicast address structure */
        memset(&mc_addr, 0, sizeof(mc_addr));
        mc_addr.sin_family      = AF_INET;
        mc_addr.sin_addr.s_addr = htonl(INADDR_ANY);
        mc_addr.sin_port        = htons(mc_port);
        
        /* bind to multicast address to socket */
        if ((bind(sock, (struct sockaddr *) &mc_addr,
                  sizeof(mc_addr))) < 0) {
            [delegate multicastDaemonDidFinish:self withError:@"bind() failed"];

            error = 1;
        }
        
        /* construct an IGMP join request structure */
        mc_req.imr_multiaddr.s_addr = inet_addr(mc_addr_str);
        mc_req.imr_interface.s_addr = htonl(INADDR_ANY);
        
        /* send an ADD MEMBERSHIP message via setsockopt */
        if ((setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,
                        (void*) &mc_req, sizeof(mc_req))) < 0) {
            [delegate multicastDaemonDidFinish:self withError:@"setsockopt() failed"];

            error = 1;
        }
        
        while  (error == 0) {          /* loop forever */
            
            
            /* clear the receive buffers & structs */
            memset(recv_str, 0, sizeof(recv_str));
            from_len = sizeof(from_addr);
            memset(&from_addr, 0, from_len);
            NSLog(@"Waiting for Packet");
            /* block waiting to receive a packet */
            if ((recv_len = recvfrom(sock, recv_str, MAX_LEN, 0,
                                     (struct sockaddr*)&from_addr, &from_len)) < 0) {
                [delegate multicastDaemonDidReceiveData:self withError:@"recvfrom() failed"];

                //error = 1;
            }
            receivedString = [NSString stringWithFormat:@"%s", recv_str];
            [delegate multicastDaemonDidReceiveData:self receivedString:receivedString];
            
            /* output received string */
            //printf("Received %d bytes from %s: ", recv_len,
                 //  inet_ntoa(from_addr.sin_addr));
            //printf("%s", recv_str);
        }
        
        /* send a DROP MEMBERSHIP message via setsockopt */
        if ((setsockopt(sock, IPPROTO_IP, IP_DROP_MEMBERSHIP,
                        (void*) &mc_req, sizeof(mc_req))) < 0) {
            [delegate multicastDaemonDidFinish:self withError:@"setsockopt() failed"];

            error = 1;
        }
        
        
        close(sock);	

        if ( error == 0) {
            [delegate multicastDaemonDidFinish:self];
        }
    }
}

@end
