//
//  MulticastReceive.m
//  WrestlingTools
//
//  Created by Adam Fenn on 11/15/11.
//  Copyright 2011 Wrestling Video Solutions. All rights reserved.
//


/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "MulticastReceive.h"
#import "MulticastDaemon.h"

#define	kQCPlugIn_Name				@"Multicast Receieve"
#define	kQCPlugIn_Description		@"Listens to a provided multicast address and port, and provides any output strings heard on that port"

@implementation MulticastReceive

/*
 Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
 @dynamic inputFoo, outputBar;
 */

@dynamic inputMulticastAddress, inputPortNumber, outputString;
@synthesize latestString, multicastAddress, portNumber, started, lastString, daemonThread, multicastDaemon;

+ (NSDictionary*) attributes
{
	/*
	 Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	 */
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	 Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	 */
	
	if([key isEqualToString:@"inputMulticastAddress"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
                @"Multicast Address", QCPortAttributeNameKey,
                nil];
	if([key isEqualToString:@"inputPortNumber"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
                @"Port Number", QCPortAttributeNameKey,
                nil];	
    if([key isEqualToString:@"outputString"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
                @"String", QCPortAttributeNameKey,
                nil];	
	
	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	 Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	 */
	
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	 Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	 */
	
	return kQCPlugInTimeModeIdle;
}

- (id) init
{
	if(self = [super init]) {
		/*
		 Allocate any permanent resource required by the plug-in.
		 */
	}
	
	return self;
}

- (void) finalize
{
	/*
	 Release any non garbage collected resources created in -init.
	 */
	
	[super finalize];
}

- (void) dealloc
{
	/*
	 Release any resources created in -init.
	 */
	//[multicastDaemon release];
	[super dealloc];
}

- (void)multicastDaemonDidStart:(MulticastDaemon*)theThreadLogicContainer
{
    latestString = nil;
}

- (void)multicastDaemonDidReceiveData:(MulticastDaemon*)theThreadLogicContainer receivedString:(NSString*) receivedString;
{
    NSLog(@"Received Data: %@", receivedString);
    latestString = [NSString stringWithString: receivedString];
}

- (void)multicastDaemonDidFinish:(MulticastDaemon*)theThreadLogicContainer
{
    // doesn't finish
}

- (void)multicastDaemonDidFinish:(MulticastDaemon*)theThreadLogicContainer withError:(NSString*) error;
{
    NSLog(@"Multicast Receive Exited With Error: %@", error);
}

- (void)multicastDaemonDidReceiveData:(MulticastDaemon*)theThreadLogicContainer withError:(NSString*) error;
{
    NSLog(@"Multicast Receive received with error: %@", error);
}

@end

@implementation MulticastReceive (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	 Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	 */
    
    self.started = FALSE;
    multicastDaemon = [[MulticastDaemon alloc] init];
    daemonThread = [NSThread alloc];

    
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	 */
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	/*
	 Called by Quartz Composer whenever the plug-in instance needs to execute.
	 Only read from the plug-in inputs and produce a result (by writing to the plug-in outputs or rendering to the destination OpenGL context) within that method and nowhere else.
	 Return NO in case of failure during the execution (this will prevent rendering of the current frame to complete).
	 
	 The OpenGL context for rendering can be accessed and defined for CGL macros using:
	 CGLContextObj cgl_ctx = [context CGLContextObj];
	 */
    
    // try to make this more efficient by not changing anything if nothing is changed
    
    
    

    
    
    // if inputs changed - kill & restart multicastDaemon
    
    // if multicastDaemon hasn't started yet, start it
    
    // if latestString != oldString  && latestString != nil then write the output
    
    // what if my values are blank?  it will error out?
    
    
    if (self.started == FALSE) {
        // set started to TRUE
        self.started = TRUE;
        
        // start the daemon for the first time
        self.multicastAddress = [NSString stringWithString:self.inputMulticastAddress];
        self.portNumber = [NSString stringWithString:self.inputPortNumber];
        NSLog(@"Multicast Address: %@:%i", self.multicastAddress, [self.portNumber intValue]);
        
        // make sure port number is good before starting
        if ( [self.portNumber intValue] < 1024 || [self.portNumber intValue] > 65535 ) {
            NSLog(@"Multicast Receive port must be between 1024 and 65535");
        } else {
            [daemonThread initWithTarget:multicastDaemon selector:@selector(doWorkWithDelegate:) object:self];
            [daemonThread start];
        }
                
    }  else {
        
        if ([self.inputPortNumber isEqualToString:portNumber] && [self.inputMulticastAddress isEqualToString:multicastAddress]) {
            // no config changes, so just check for new data
            if ( [ self.latestString isEqualToString:self.lastString] || self.latestString == nil) {
                // do nothing, no new data
            } else {
                NSLog(@"we have new data: %@, %@", self.lastString, self.latestString);
                self.outputString = [NSString stringWithString:self.latestString];
                self.lastString = [NSString stringWithString:self.latestString];
            }
        } else {
            // there was a change in config, so restart the daemon
            NSLog(@"input changes made");
            self.multicastAddress = [NSString stringWithString:self.inputMulticastAddress];
            self.portNumber = [NSString stringWithString:self.inputPortNumber];
            NSLog(@"here");
            if ( [daemonThread isExecuting] ) {  // this line is crashing the system
                // could be that it died, or that it wasn't started to begin with
                NSLog(@"daemon is running, need to restart");
                [daemonThread cancel];
            }
            NSLog(@"after cancel");
            //[daemonThread start];
        }
    }
    

 
    
    
 
return YES;
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	 */
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	 */
}

@end
