#import <Foundation/Foundation.h>
#include <stdio.h>      /* for fprintf() */
#include <sys/socket.h> /* for socket(), connect(), send(), and recv() */
#include <arpa/inet.h>  /* for sockaddr_in and inet_addr() */
#include <stdlib.h>     /* for atoi() and exit() */
#include <string.h>     /* for memset() */
#include <unistd.h>     /* for sleep() */

//void DieWithError(char *errorMessage);  /* External error handling function */

int main (int argc, const char * argv[]) {
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];

    // insert code here...
	int sock;                         /* Socket */
	struct sockaddr_in multicastAddr; /* Multicast address */
	char *multicastIP;                /* IP Multicast address */
	unsigned short multicastPort;     /* Server port */
	char *sendString;                 /* String to multicast */
	unsigned char multicastTTL;       /* TTL of multicast packets */
	unsigned int sendStringLen;       /* Length of string to multicast */
	
	if ((argc < 4) || (argc > 5))         /* Test for correct number of parameters */
	{
		fprintf(stderr,"Usage:  %s <Multicast Address> <Port> <Send String> [<TTL>]\n",
						argv[0]);
		exit(1);
	}
	
	multicastIP = argv[1];            /* First arg:  multicast IP address */ 
	multicastPort = atoi(argv[2]);    /* Second arg:  multicast port */
	//sendString = argv[3];             /* Third arg:  String to multicast */
	
	if (argc == 5)                     /* Is TTL specified on command-line? */
		multicastTTL = atoi(argv[4]);  /* Command-line specified TTL */
	else
		multicastTTL = 1;              /* Default TTL = 1 */
	
	/* Create socket for sending/receiving datagrams */
	sock = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
		//DieWithError("socket() failed");
	
	/* Set TTL of multicast packet */
	setsockopt(sock, IPPROTO_IP, IP_MULTICAST_TTL, (void *) &multicastTTL, 
						 sizeof(multicastTTL));
		//DieWithError("setsockopt() failed");
	
	/* Construct local address structure */
	memset(&multicastAddr, 0, sizeof(multicastAddr));   /* Zero out structure */
	multicastAddr.sin_family = AF_INET;                 /* Internet address family */
	multicastAddr.sin_addr.s_addr = inet_addr(multicastIP);/* Multicast IP address */
	multicastAddr.sin_port = htons(multicastPort);         /* Multicast port */
	int q=0;
	NSString *msg;
	for (;;) {
		q++;
		msg = [NSString stringWithFormat:@"%i %s", q, argv[3]];
		sendString = [msg UTF8String];
		NSLog(@"%@", msg);
		//const char *x = [msg cStringUsingEncoding:NSUTF32BigEndianStringEncoding];
		//int len = [msg lengthOfBytesUsingEncoding:NSUTF32BigEndianStringEncoding];
		
		sendStringLen = strlen(sendString);  /* Find length of sendString */
		
		/*for (int i = 0; i < len; i++)
		 printf ("0x%x\n", x[i]);*/
		
		
		/* Multicast sendString in datagram to clients every 3 seconds */
		sendto(sock, sendString, sendStringLen, 0, (struct sockaddr *) 
					 &multicastAddr, sizeof(multicastAddr));
		usleep(1000000);
	}
	
	[pool drain];
    return 0;
}
